package domain_checker

import (
	"code.mrmelon54.com/melon/summer/pkg/tables/web"
	"code.mrmelon54.com/melon/summer/pkg/utils"
	"log"
	"strings"
	"sync"
	"xorm.io/xorm"
)

func New(db *xorm.Engine) *DomainChecker {
	d := &DomainChecker{
		db:         db,
		domainSync: &sync.RWMutex{},
		domainMap:  make(map[string]struct{}),
	}
	d.Compile()
	return d
}

type DomainChecker struct {
	db         *xorm.Engine
	domainSync *sync.RWMutex
	domainMap  map[string]struct{}
}

func (d *DomainChecker) Compile() {
	go func() {
		domainMap := make(map[string]struct{})
		err := d.internalCompile(domainMap)
		if err != nil {
			// log compile errors
			log.Printf("[DomainChecker] Compile failed: %s\n", err)
			return
		}
		// lock while replacing the map
		d.domainSync.Lock()
		d.domainMap = domainMap
		d.domainSync.Unlock()
	}()
}

func (d *DomainChecker) internalCompile(domainMap map[string]struct{}) error {
	log.Println("[DomainChecker] Pulling from database")
	var domains []web.Domain
	err := d.db.Find(&domains)
	if err != nil {
		return err
	}
	for _, i := range domains {
		domainMap[i.Domain] = struct{}{}
	}

	// well I guess we are done
	log.Printf("[DomainChecker] Finished compiling, %d domains will be available shortly\n", len(domains))
	return nil
}

func (d *DomainChecker) IsValid(host string) bool {
	domain, ok := utils.GetDomainWithoutPort(host)
	if !ok {
		return false
	}
	d.domainSync.RLock()
	defer d.domainSync.RUnlock()
	n := strings.Split(domain, ".")
	for i := 0; i < len(n); i++ {
		if _, ok := d.domainMap[strings.Join(n[i:], ".")]; ok {
			return true
		}
	}
	return false
}
