package quick_db

import (
	"code.mrmelon54.com/melon/summer/pkg/tables/certificate"
	"code.mrmelon54.com/melon/summer/pkg/tables/web"
	"code.mrmelon54.com/melon/summer/pkg/utils"
	_ "github.com/mattn/go-sqlite3"
	"github.com/stretchr/testify/assert"
	"testing"
	"xorm.io/xorm"
)

func TestQuickCertDebug(t *testing.T) {
	db, err := xorm.NewEngine("sqlite3", ":memory:")
	assert.NoError(t, err)
	assert.NoError(t, db.CreateTables(&certificate.Certificate{}, &certificate.CertificateData{}, &certificate.CertificateDomain{}, &web.Domain{}, &web.ApiDomain{}, &web.ApiRoute{}))
	QuickDb(db)

	var c []certificate.Certificate
	var c2 []certificate.CertificateData
	var c3 []certificate.CertificateDomain
	assert.NoError(t, db.Find(&c))
	assert.NoError(t, db.Find(&c2))
	assert.NoError(t, db.Find(&c3))

	assert.Equal(t, 1, len(c))
	assert.Equal(t, 1, len(c2))
	assert.Equal(t, 2, len(c3))

	c[0].Id = 0
	assert.Equal(t, certificate.Certificate{
		Owner:       1,
		LetsEncrypt: 1,
		Namesilo:    1,
		AutoRenew:   utils.PBool(false),
		Active:      utils.PBool(true),
		Renewing:    utils.PBool(false),
		RenewFailed: utils.PBool(false),
	}, c[0])
	c3[0].DomainId = 0
	c3[0].CertId = 0
	c3[1].DomainId = 0
	c3[1].CertId = 0
	assert.Equal(t, certificate.CertificateDomain{
		Domain:   "summer.test",
		Wildcard: utils.PBool(false),
	}, c3[0])
	assert.Equal(t, certificate.CertificateDomain{
		Domain:   "summer.test",
		Wildcard: utils.PBool(true),
	}, c3[1])
}
