package routing

import (
	"code.mrmelon54.com/melon/summer/pkg/utils"
	"net/http"
	"net/url"
	"path"
	"strings"
)

var redirectCodeMap = map[string]int{
	"temp": http.StatusTemporaryRedirect,
	"perm": http.StatusPermanentRedirect,
}

type ServeRedirect struct {
	Target string
	Path   string
	Prefix string
	Flags  []string
}

func (s ServeRedirect) ServeHTTP(rw http.ResponseWriter, req *http.Request) {
	if s.Prefix == "/" {
		s.Prefix = ""
	}
	var p2 string
	if s.Path == "" {
		p2 = req.URL.Path
	} else {
		if !strings.HasPrefix(req.URL.Path, s.Prefix) {
			utils.RespondHttpStatus(rw, http.StatusExpectationFailed)
			return
		}
		p2 = path.Join(s.Path, req.URL.Path[len(s.Prefix):])
		if strings.HasSuffix(req.URL.Path, "/") && !strings.HasSuffix(p2, "/") {
			p2 += "/"
		}
	}
	if !strings.HasPrefix(p2, "/") {
		p2 = "/" + p2
	}
	u := url.URL{Scheme: "https", Host: s.Target, Path: p2}

	// Update temp/perm status code with flag
	c := http.StatusFound
	if s.Flags != nil {
	outerLoop:
		for k, v := range redirectCodeMap {
			for _, i := range s.Flags {
				if k == i {
					c = v
					break outerLoop
				}
			}
		}
	}
	http.Redirect(rw, req, u.String(), c)
}
