package routing

import (
	"github.com/stretchr/testify/assert"
	httpTest "github.com/stretchr/testify/http"
	"net/http"
	"testing"
)

func TestServeRedirect_Location(t *testing.T) {
	redirectLocationCheck(t, "https://b.example.com/", "https://a.example.com", ServeRedirect{
		Target: "b.example.com",
		Path:   "/",
		Prefix: "/",
		Flags:  nil,
	})
	redirectLocationCheck(t, "https://b.example.com/", "https://a.example.com", ServeRedirect{
		Target: "b.example.com",
		Path:   "",
		Prefix: "/",
		Flags:  nil,
	})
	redirectLocationCheck(t, "https://b.example.com/world", "https://a.example.com/hello", ServeRedirect{
		Target: "b.example.com",
		Path:   "/world",
		Prefix: "/hello",
		Flags:  nil,
	})
	redirectLocationCheck(t, "https://b.example.com/goodbye/world", "https://a.example.com/hello/world", ServeRedirect{
		Target: "b.example.com",
		Path:   "/goodbye",
		Prefix: "/hello",
		Flags:  []string{"prefix"},
	})
}

func TestServeRedirect_StatusCode(t *testing.T) {
	redirectCodeCheck(t, http.StatusFound, ServeRedirect{
		Target: "a.example.com",
		Path:   "",
		Prefix: "",
		Flags:  nil,
	})
	redirectCodeCheck(t, http.StatusFound, ServeRedirect{
		Target: "a.example.com",
		Path:   "",
		Prefix: "",
		Flags:  []string{},
	})
	redirectCodeCheck(t, http.StatusTemporaryRedirect, ServeRedirect{
		Target: "a.example.com",
		Path:   "",
		Prefix: "",
		Flags:  []string{"temp"},
	})
	redirectCodeCheck(t, http.StatusPermanentRedirect, ServeRedirect{
		Target: "a.example.com",
		Path:   "",
		Prefix: "",
		Flags:  []string{"perm"},
	})
}

func redirectLocationCheck(t *testing.T, expected string, input string, serve ServeRedirect) {
	rw := &httpTest.TestResponseWriter{}
	req, err := http.NewRequest(http.MethodGet, input, nil)
	assert.NoError(t, err)
	serve.ServeHTTP(rw, req)
	assert.Equal(t, expected, rw.Header().Get("Location"))
}

func redirectCodeCheck(t *testing.T, expected int, serve ServeRedirect) {
	rw := &httpTest.TestResponseWriter{}
	req, err := http.NewRequest(http.MethodGet, "https://example.com", nil)
	assert.NoError(t, err)
	serve.ServeHTTP(rw, req)
	assert.Equal(t, expected, rw.StatusCode)
}
