package servers

import (
	"code.mrmelon54.com/melon/summer/pkg/api"
	"code.mrmelon54.com/melon/summer/pkg/api/crud"
	"code.mrmelon54.com/melon/summer/pkg/tables/web"
	"code.mrmelon54.com/melon/summer/pkg/utils"
	"github.com/gorilla/mux"
	"github.com/mrmelon54/mjwt"
	"log"
	"net/http"
	"time"
	"xorm.io/xorm"
)

func NewApiServer(listen string, db *xorm.Engine, verify mjwt.Provider, compileTarget api.MultiCompilable) *http.Server {
	router := mux.NewRouter()
	crud.NewCrudHandler[web.HttpService](router, verify, crud.NewSinglePermProvider[web.HttpService]("manage:http-service", crud.NewDatabaseProvider[web.HttpService](db, "id")), "/http-services", "/http-service/{id}")
	crud.NewCrudHandler[web.HttpRedirect](router, verify, crud.NewSinglePermProvider[web.HttpRedirect]("manage:http-redirect", crud.NewDatabaseProvider[web.HttpRedirect](db, "id")), "/http-redirects", "/http-redirect/{id}")
	crud.NewCrudHandler[web.ApiRoute](router, verify, crud.NewSinglePermProvider[web.ApiRoute]("manage:api-route", crud.NewDatabaseProvider[web.ApiRoute](db, "id")), "/api-routes", "/api-route/{id}")
	router.HandleFunc("/compile", func(rw http.ResponseWriter, req *http.Request) {
		compileTarget.Compile()
		rw.WriteHeader(http.StatusAccepted)
	}).Methods(http.MethodPost)

	s := &http.Server{
		Addr:              listen,
		Handler:           router,
		ReadTimeout:       1 * time.Minute,
		ReadHeaderTimeout: 1 * time.Minute,
		WriteTimeout:      1 * time.Minute,
		IdleTimeout:       1 * time.Minute,
		MaxHeaderBytes:    2500,
	}
	log.Printf("[API] Starting API server on: '%s'\n", s.Addr)
	go utils.RunBackgroundHttp("API", s)
	return s
}
