package servers

import (
	domainChecker "code.mrmelon54.com/melon/summer/cmd/azalea/domain-checker"
	"code.mrmelon54.com/melon/summer/pkg/tables/renewal"
	"code.mrmelon54.com/melon/summer/pkg/utils"
	"fmt"
	"github.com/gorilla/mux"
	"log"
	"net/http"
	"time"
	"xorm.io/xorm"
)

func NewHttpServer(listen string, httpsPort uint16, db *xorm.Engine, domainCheck *domainChecker.DomainChecker) *http.Server {
	router := mux.NewRouter()
	router.HandleFunc("/.well-known/acme-challenge/{token}", func(rw http.ResponseWriter, req *http.Request) {
		vars := mux.Vars(req)
		if hostname, ok := utils.GetDomainWithoutPort(req.Host); ok {
			if !domainCheck.IsValid(req.Host) {
				utils.RequestKiller(rw)
				return
			}
			if tokenValue, ok := vars["token"]; ok {
				rw.WriteHeader(http.StatusOK)
				content := renewal.AcmeContent{
					Domain: hostname,
					Key:    tokenValue,
				}
				ok, err := db.Asc("id").Limit(1).Get(&content)
				if err == nil && ok {
					_, _ = rw.Write([]byte(content.Content))
					return
				} else if err != nil {
					log.Printf("[HTTP] Error finding acme content: '%s': %s\n", tokenValue, err)
				}
				_, _ = rw.Write([]byte("invalid acme key"))
				return
			}
		}
	})
	router.NotFoundHandler = http.HandlerFunc(func(rw http.ResponseWriter, req *http.Request) {
		if h, ok := utils.GetDomainWithoutPort(req.Host); ok {
			if httpsPort != 443 {
				req.URL.Host = fmt.Sprintf("%s:%d", h, httpsPort)
			} else {
				req.URL.Host = h
			}
			req.URL.Scheme = "https"
			http.Redirect(rw, req, req.URL.String(), http.StatusTemporaryRedirect)
			return
		}
		_, _ = rw.Write([]byte("500 Internal Server Error"))
	})

	s := &http.Server{
		Addr:              listen,
		Handler:           router,
		ReadTimeout:       1 * time.Minute,
		ReadHeaderTimeout: 1 * time.Minute,
		WriteTimeout:      1 * time.Minute,
		IdleTimeout:       1 * time.Minute,
		MaxHeaderBytes:    2500,
	}
	log.Printf("[HTTP] Starting HTTP server on: '%s'\n", s.Addr)
	go utils.RunBackgroundHttp("HTTP", s)
	return s
}
