package api

import (
	"code.mrmelon54.com/melon/summer/pkg/api"
	"code.mrmelon54.com/melon/summer/pkg/tables/certificate"
	"encoding/json"
	"github.com/gorilla/mux"
	"net/http"
	"time"
	"xorm.io/xorm"
)

type CertPublic struct {
	Id          uint64    `json:"id"`
	NotAfter    time.Time `json:"notAfter"`
	Domain      string    `json:"domain"`
	Certificate string    `json:"certificate"`
}

func handleGetPublic(router *mux.Router, engine *xorm.Engine, p string) {
	router.HandleFunc(p, func(rw http.ResponseWriter, req *http.Request) {
		vars := mux.Vars(req)
		s, ok := vars["domain"]
		if !ok {
			api.GenericErrorMsg[certificate.CertificateMetaDomainJoiner](rw, api.ErrParamMissing, http.StatusBadRequest, "Parameter error")
			return
		}
		cert, err := certificate.GetCertForDomain(engine, s)
		if err != nil {
			api.GenericErrorMsg[certificate.CertificateMetaDomainJoiner](rw, err, http.StatusInternalServerError, "Database error")
			return
		}
		rw.Header().Set("Content-Type", "application/json")
		rw.WriteHeader(http.StatusOK)
		d := cert.Domain
		if cert.Wildcard != nil && *cert.Wildcard {
			d = "*." + cert.Domain
		}
		err = json.NewEncoder(rw).Encode(CertPublic{
			Id:          cert.Id,
			NotAfter:    cert.NotAfter,
			Domain:      d,
			Certificate: string(cert.CertificateBytes),
		})
		if api.GenericErrorMsg[certificate.Certificate](rw, err, http.StatusInternalServerError, "Encode error") {
			return
		}
	}).Methods(http.MethodGet)
}
