package api

import (
	"code.mrmelon54.com/melon/summer/pkg/api"
	"code.mrmelon54.com/melon/summer/pkg/api/crud"
	"code.mrmelon54.com/melon/summer/pkg/renewal"
	"github.com/gorilla/mux"
	"github.com/mrmelon54/mjwt"
	"net/http"
	"xorm.io/xorm"
)

type PermBaseConfig struct {
	Certificate     string `yaml:"certificate"`
	CertificatePair string `yaml:"certificatePair"`
}

func NewApiServer(listen string, db *xorm.Engine, renewal *renewal.Renewal, verify mjwt.Provider) *http.Server {
	router := mux.NewRouter()
	handleGetPublic(router, db, "/domain-certificate/{domain}")
	crud.NewCrudHandler[CertDbItem](router, verify, crud.NewSinglePermProvider[CertDbItem]("manage:certificate", NewCertDbProvider(db, renewal)), "/certificates", "/certificate/{id}")
	crud.NewCrudHandler[CertPrivate](router, verify, crud.NewSinglePermProvider[CertPrivate]("manage:certificate:admin", certPrivateProvider{db: db}), "/certificate-pairs", "/certificate-pair/{domain}")
	router.HandleFunc("/renew", func(rw http.ResponseWriter, req *http.Request) {
		renewal.Renew()
		rw.WriteHeader(http.StatusAccepted)
	})
	return api.RunApiServer(listen, router)
}
