package main

import (
	"code.mrmelon54.com/melon/summer/cmd/buttercup/api"
	"code.mrmelon54.com/melon/summer/pkg/cli"
	"code.mrmelon54.com/melon/summer/pkg/notifier"
	"code.mrmelon54.com/melon/summer/pkg/renewal"
	"code.mrmelon54.com/melon/summer/pkg/tables/certificate"
	renewal2 "code.mrmelon54.com/melon/summer/pkg/tables/renewal"
	"code.mrmelon54.com/melon/summer/pkg/tables/web"
	"code.mrmelon54.com/melon/summer/pkg/utils"
	"github.com/mrmelon54/mjwt"
	"log"
	"net/http"
	"sync"
)

var usedTables = []any{
	&web.Domain{},
	&renewal2.AcmeContent{},
	&renewal2.LetsEncryptAccount{},
	&renewal2.NamesiloAccount{},
	&certificate.Certificate{},
	&certificate.CertificateData{},
	&certificate.CertificateDomain{},
}

func main() {
	cli.RunWithConfig[ButtercupConfig]("buttercup", func(t ButtercupConfig) string { return t.Database }, new(Buttercup))
}

type Buttercup struct {
	conf      ButtercupConfig
	runner    *cli.Runner[ButtercupConfig]
	renewal   *renewal.Renewal
	renewalWg *sync.WaitGroup
	apiServer *http.Server
	verify    mjwt.Provider
}

func (buttercup *Buttercup) Init(runner *cli.Runner[ButtercupConfig]) {
	buttercup.runner = runner
	buttercup.conf = runner.Config
	utils.Check("[Buttercup.Init()] Failed to sync tables", runner.Database.Sync(usedTables...))
	var err error
	buttercup.verify, err = mjwt.NewMJwtVerifierFromFile(buttercup.conf.Auth.Public)
	utils.Check("[Buttercup.Init()] Failed to load verifier public key", err)
	notify, err := notifier.Init(buttercup.conf.Notify)
	utils.Check("[Buttercup.Init()] Failed to load notifier", err)
	log.Printf("[Buttercup.Init()] Using LetsEncrypt directory: %s\n", buttercup.conf.Renewal.LetsEncrypt.Directory)

	buttercup.renewalWg = &sync.WaitGroup{}
	provider := renewal.NewCustomHTTPProvider(runner.Database)
	buttercup.renewal = renewal.NewRenewalService(buttercup.renewalWg, runner.Database, provider, buttercup.conf.Renewal, notify)
	buttercup.apiServer = api.NewApiServer(buttercup.conf.Listen, runner.Database, buttercup.renewal, buttercup.verify)
}

func (buttercup *Buttercup) Destroy() {
	buttercup.renewal.Shutdown()
	buttercup.renewalWg.Wait()
	_ = buttercup.apiServer.Close()
}
