package main

import (
	"code.mrmelon54.com/melon/summer/pkg/api"
	"code.mrmelon54.com/melon/summer/pkg/api/crud"
	"code.mrmelon54.com/melon/summer/pkg/cli"
	"code.mrmelon54.com/melon/summer/pkg/tables/web"
	"code.mrmelon54.com/melon/summer/pkg/tcp"
	"code.mrmelon54.com/melon/summer/pkg/udp"
	"code.mrmelon54.com/melon/summer/pkg/utils"
	"github.com/gorilla/mux"
	"github.com/mrmelon54/mjwt"
	"net/http"
)

var usedTables = []any{
	&web.TcpRedirect{},
	&web.UdpRedirect{},
}

func main() {
	cli.RunWithConfig[RoseConfig]("rose", func(t RoseConfig) string { return t.Database }, new(Rose))
}

type Rose struct {
	conf   RoseConfig
	runner *cli.Runner[RoseConfig]
	server *http.Server
	tcpApi *tcp.Manager
	udpApi *udp.Manager
	verify mjwt.Provider
}

func (rose *Rose) Init(runner *cli.Runner[RoseConfig]) {
	rose.runner = runner
	rose.conf = runner.Config
	utils.Check("[Rose.Init()] Failed to sync tables", runner.Database.Sync(usedTables...))
	var err error
	rose.verify, err = mjwt.NewMJwtVerifierFromFile(rose.conf.Auth.Public)
	utils.Check("[Rose.Init()] Failed to load verifier public key", err)

	router := mux.NewRouter()
	rose.tcpApi = tcp.NewTcpManager(runner.Database)
	rose.udpApi = udp.NewUdpManager(runner.Database)
	crud.NewCrudHandler[web.TcpRedirect](router, rose.verify, crud.NewSinglePermProvider[web.TcpRedirect]("manager:tcp", crud.NewDatabaseProvider[web.TcpRedirect](runner.Database, "id")), "/tcp", "/tcp/{id}")
	crud.NewCrudHandler[web.UdpRedirect](router, rose.verify, crud.NewSinglePermProvider[web.UdpRedirect]("manager:udp", crud.NewDatabaseProvider[web.UdpRedirect](runner.Database, "id")), "/udp", "/udp/{id}")
	router.HandleFunc("/compile", func(rw http.ResponseWriter, req *http.Request) {
		rose.tcpApi.Compile()
		rose.udpApi.Compile()
	})
	rose.server = api.RunApiServer(rose.conf.Listen, router)
}

func (rose *Rose) Destroy() {
	_ = rose.server.Close()
	rose.tcpApi.Destroy()
	rose.udpApi.Destroy()
}
