package account_server

import (
	"regexp"
	"strings"
)

var (
	passwordPattern   = regexp.MustCompile("^.{8,32}$")
	passwordHasDigit  = regexp.MustCompile("\\d")
	passwordHasSymbol = regexp.MustCompile("\\W")

	allPasswordPatterns = []func(string) bool{
		passwordPattern.MatchString,
		passwordHasDigit.MatchString,
		passwordHasSymbol.MatchString,

		// has a lowercase character if `ToUpper()` result is different
		func(s string) bool {
			return strings.ToUpper(s) != s
		},

		// has an uppercase character if `ToLower()` result is different
		func(s string) bool {
			return strings.ToLower(s) != s
		},
	}
)

func ValidatePassword(pw string) bool {
	for _, i := range allPasswordPatterns {
		if !i(pw) {
			return false
		}
	}
	return true
}
