package account_server

import (
	"code.mrmelon54.com/melon/summer/pkg/api/crud"
	"code.mrmelon54.com/melon/summer/pkg/claims/auth"
	"code.mrmelon54.com/melon/summer/pkg/tables/user"
	"fmt"
	"time"
	"xorm.io/xorm"
)

type userProvider struct {
	db     *xorm.Engine
	dbCrud crud.CheckedProvider[user.User]
}

func NewUserProvider(db *xorm.Engine) crud.Provider[user.User] {
	return &userProvider{db, crud.NewDatabaseProvider[user.User](db, "id")}
}

func (u userProvider) Find(claims *auth.AccessTokenClaims) ([]user.User, error) {
	if !claims.Perms.Has("super:admin") {
		return nil, crud.ErrForbidden
	}
	return u.dbCrud.Find()
}

func (u userProvider) Add(_ *auth.AccessTokenClaims, _ user.User) (user.User, error) {
	// TODO: implement register instead of this
	return user.User{}, crud.ErrMethodNotAllowed
}

func (u userProvider) Get(claims *auth.AccessTokenClaims, id string) (user.User, bool, error) {
	if claims == nil {
		return user.User{}, false, crud.ErrForbidden
	}
	myId := fmt.Sprint(claims.UserId)
	if id == "@me" || id == myId {
		return u.dbCrud.Get(myId)
	}
	if claims.Perms.Has("super:admin") {
		return u.dbCrud.Get(id)
	}
	get, ok, err := u.dbCrud.Get(id)
	if ok && err == nil {
		if user.ParseVisibility(get.Visibility) == user.ProfileVisibilityPublic {
			z := get
			z.Email = "melon@mrmelon54.com"
			z.Visibility = nil
			z.Gender = nil
			z.Birthday = time.UnixMilli(-1)
			z.LastOnline = time.UnixMilli(-1)
			z.EmailVerified = nil
			z.Enabled = nil
			return get, true, err
		}
	}
	return user.User{}, false, crud.ErrForbidden
}

func (u userProvider) Put(claims *auth.AccessTokenClaims, id string, data user.User) error {
	if claims == nil {
		return crud.ErrForbidden
	}
	myId := fmt.Sprint(claims.UserId)
	if id == "@me" || id == myId {
		return u.dbCrud.Put(id, data.ClearForPut())
	}
	if claims.Perms.Has("super:admin") {
		return u.dbCrud.Put(id, data.ClearForAdminPut())
	}
	return crud.ErrForbidden
}

func (u userProvider) Patch(_ *auth.AccessTokenClaims, _ string, _ user.User) error {
	// TODO: implement patch
	return crud.ErrMethodNotAllowed
}

func (u userProvider) Delete(_ *auth.AccessTokenClaims, _ string) error {
	// TODO: implement delete
	return crud.ErrMethodNotAllowed
}
