package oauth_server

import (
	"code.mrmelon54.com/melon/summer/pkg/api/crud"
	"code.mrmelon54.com/melon/summer/pkg/claims/auth"
	"code.mrmelon54.com/melon/summer/pkg/tables/oauth"
	"xorm.io/xorm"
)

type oAuthAppProvider struct {
	db     *xorm.Engine
	dbCrud crud.CheckedProvider[oauth.App]
}

func NewOAuthAppProvider(db *xorm.Engine) crud.Provider[oauth.App] {
	return &oAuthAppProvider{db, crud.NewDatabaseProvider[oauth.App](db, "app_id")}
}

func (o oAuthAppProvider) Find(_ *auth.AccessTokenClaims) ([]oauth.App, error) {
	// TODO: implement this for users and admins
	return nil, crud.ErrMethodNotAllowed
}

func (o oAuthAppProvider) Add(_ *auth.AccessTokenClaims, item oauth.App) (oauth.App, error) {
	return o.dbCrud.Add(item)
}

func (o oAuthAppProvider) Get(_ *auth.AccessTokenClaims, id string) (oauth.App, bool, error) {
	var a oauth.App
	ok, err := o.db.Where("app_id = ? and enabled = ?", id, true).Get(&a)
	return a, ok, err
}

func (o oAuthAppProvider) Put(claims *auth.AccessTokenClaims, id string, item oauth.App) error {
	_, err := o.db.Where("app_id = ? and user_id = ?", id, claims.UserId).Update(&item)
	return err
}

func (o oAuthAppProvider) Patch(claims *auth.AccessTokenClaims, id string, item oauth.App) error {
	_, err := o.db.Where("app_id = ? and user_id = ?", id, claims.UserId).Update(&item)
	return err
}

func (o oAuthAppProvider) Delete(_ *auth.AccessTokenClaims, _ string) error {
	return crud.ErrMethodNotAllowed
}
