package tcp

import (
	"code.mrmelon54.com/melon/summer/pkg/utils"
	"log"
	"net"
)

type Proxy struct {
	lAddr *net.TCPAddr
	rAddr *net.TCPAddr
	done  *utils.DoneChan
	l     *net.TCPListener
	n     uint64
}

func NewTcpProxy(lAddr, rAddr *net.TCPAddr) (*Proxy, error) {
	p := &Proxy{
		lAddr: lAddr,
		rAddr: rAddr,
		done:  utils.NewDoneChan(),
	}
	l, err := net.ListenTCP("tcp", p.lAddr)
	if err != nil {
		return nil, err
	}
	p.l = l
	go p.listen()
	return p, nil
}

func (p *Proxy) Quit() {
	p.done.Close()
	_ = p.l.Close()
}

func (p *Proxy) listen() {
	for {
		accept, err := p.l.AcceptTCP()
		if err != nil {
			if p.done.Running() {
				log.Printf("[WARN @ %s] Error accepting connection: '%s'\n", p.lAddr, err)
				continue
			}
			break
		}
		p.setupConn(accept)
	}
}

func (p *Proxy) setupConn(accept *net.TCPConn) {
	defer func(accept *net.TCPConn) {
		_ = accept.Close()
	}(accept)

	dialConn, err := net.DialTCP("tcp", nil, p.rAddr)
	if err != nil {
		return
	}
	errSig := utils.NewDoneChan()

	go p.pipeConn(errSig, accept.RemoteAddr(), accept, dialConn)
	go p.pipeConn(errSig, accept.RemoteAddr(), dialConn, accept)

	<-errSig.C
	_ = accept.SetLinger(0)
	_ = accept.CloseRead()
}

func (p *Proxy) pipeConn(sig *utils.DoneChan, addr net.Addr, a, b net.Conn) {
	buf := make([]byte, 1024)
	for {
		// read data
		i, err := a.Read(buf)
		if err != nil {
			log.Printf("[WARN %s @ %s] Read failed '%s'\n", addr, p.lAddr, err)
			break
		}
		data := buf[:i]

		// write data
		_, err = b.Write(data)
		if err != nil {
			log.Printf("[WARN %s @ %s] Write failed '%s'\n", addr, p.lAddr, err)
			break
		}
	}
	sig.Close()
}
